#ifndef LNM_HTTP_ROUTER
#define LNM_HTTP_ROUTER

#define LNM_HTTP_MAX_KEY_SEGMENTS 4

#include "lnm/common.h"
#include "lnm/http/consts.h"

typedef struct lnm_http_route lnm_http_route;

typedef struct lnm_http_route_match_segment {
  size_t start;
  size_t len;
} lnm_http_route_match_segment;

typedef struct lnm_http_route_match {
  const lnm_http_route *route;
  lnm_http_method method;
  lnm_http_route_match_segment key_segments[LNM_HTTP_MAX_KEY_SEGMENTS];
} lnm_http_route_match;

typedef struct lnm_http_router lnm_http_router;

typedef enum lnm_http_route_err {
  lnm_http_route_err_match = 0,
  lnm_http_route_err_unknown_route = 1,
  lnm_http_route_err_unknown_method = 2,
} lnm_http_route_err;

/**
 * Allocate and initialize a new http_router.
 */
lnm_err lnm_http_router_init(lnm_http_router **out);

void lnm_http_router_free(lnm_http_router *router);

lnm_err lnm_http_router_add(lnm_http_route **out, lnm_http_router *http_router,
                            lnm_http_method method, const char *path);

/**
 * Add all of the child router's routes to the parent router, under the given
 * route prefix.
 */
lnm_err lnm_http_router_nest(lnm_http_router *parent,
                             const lnm_http_router *child, const char *prefix);

lnm_http_route_err lnm_http_router_route(lnm_http_route_match *out,
                                         const lnm_http_router *router,
                                         lnm_http_method method,
                                         const char *path);

lnm_err lnm_http_route_match_get(lnm_http_route_match_segment **out,
                                 lnm_http_route_match *match, const char *key);

#endif
